@@ -91,8 +91,14 @@ class Dedimania(AppConfig):
 			self.setting_chat_welcome, self.setting_chat_announce, self.setting_sent_announce
 		)
 
+		# Load initial data.
+		if not self.widget:
+			self.widget = DedimaniaRecordsWidget(self)
+
 		# Load settings + initiate api.
-		await self.reload_settings()
+		if self.instance.game.game == "tm":
+			await self.reload_settings()
+			return
 
 		# Register signals
 		self.context.signals.listen(mp_signals.map.map_begin, self.map_begin)
@@ -112,11 +118,6 @@ class Dedimania(AppConfig):
 			).add_param('record', required=False, type=int, help='Custom record rank to compare with. Defaults to 1.', default=1)
 		)
 
-		# Change round results widget location.
-
-		# Load initial data.
-		self.widget = DedimaniaRecordsWidget(self)
-
 	async def reload_settings(self, *args, **kwargs):
 		# Check setting + return errors if not correct!
 		self.login = await self.setting_server_login.get_value(refresh=True) or self.instance.game.server_player_login
